@@ -85,7 +85,7 @@ extern char *	getenv();
 __libc_lock_define (extern, __libc_localtime_lock);
 __libc_lock_define (extern, __libc_gmtime_lock);
 
-extern time_t	time();
+extern time_t	time(time_t *);
 
 #define ACCESS_MODE	O_RDONLY
 #define OPEN_MODE	O_RDONLY
@@ -161,33 +161,48 @@ struct rule {
 ** Prototypes for static functions.
 */
 
-static long		detzcode P((const char * codep));
-static const char *	getzname P((const char * strp));
-static const char *	getnum P((const char * strp, int * nump, int min,
-				int max));
-static const char *	getsecs P((const char * strp, long * secsp));
-static const char *	getoffset P((const char * strp, long * offsetp));
-static const char *	getrule P((const char * strp, struct rule * rulep));
-static void		gmtload P((struct state * sp));
-static void		gmtsub P((const time_t * timep, long offset,
-				struct tm * tmp));
-static void		localsub P((const time_t * timep, long offset,
-				struct tm * tmp));
-static void		normalize P((int * tensptr, int * unitsptr, int base));
-static void		settzname P((void));
-static time_t		time1 P((struct tm * tmp, void (* funcp)(),
-				long offset));
-static time_t		time2 P((struct tm *tmp, void (* funcp)(),
-				long offset, int * okayp));
-static void		timesub P((const time_t * timep, long offset,
-				const struct state * sp, struct tm * tmp));
-static int		tmcomp P((const struct tm * atmp,
-				const struct tm * btmp));
-static time_t		transtime P((time_t janfirst, int year,
-				const struct rule * rulep, long offset));
-static int		tzload P((const char * name, struct state * sp));
-static int		tzparse P((const char * name, struct state * sp,
-				int lastditch));
+static long	         detzcode    P((const char *));
+static const char   *getzname    P((const char *));
+static const char   *getnum      P((const char *, int *, int, int));
+static const char   *getsecs     P((const char * strp, long * secsp));
+static const char   *getoffset   P((const char * strp, long * offsetp));
+static const char   *getrule     P((const char * strp, struct rule * rulep));
+static void          gmtload     P((struct state * sp));
+static void          gmtsub      P((const time_t * timep, long offset,
+                                    struct tm * tmp));
+static void          localsub    P((const time_t * timep, long offset,
+                                    struct tm * tmp));
+static void          normalize   P((int * tensptr, int * unitsptr, int base));
+static void          settzname   P((void));
+static void          timesub     P((const time_t * timep, long offset,
+                                    const struct state * sp, struct tm * tmp));
+static time_t        transtime   P((time_t janfirst, int year,
+                                    const struct rule * rulep, long offset));
+static int           tzload      P((const char * name, struct state * sp));
+static int           tzparse     P((const char * name, struct state * sp,
+                                    int lastditch));
+void                 tzsetwall   P((void));
+struct tm           *localtime   P((const time_t *const));
+struct tm           *localtime_r P((const time_t *const, struct tm *));
+struct tm           *gmtime_r    P((const time_t *const, struct tm *));
+char                *asctime_r   P((register const struct tm *, char	*));
+char                *asctime_r   P((register const struct tm *timeptr,
+                                    char *result));
+char                *asctime     P((register const struct tm *timeptr));
+char                *ctime       P((const time_t * const	timep));
+char                *ctime_r     P((const time_t * const timep, char *result));
+static void          normalize   P((int * const tensptr, int * const unitsptr,
+                                    const int base));
+static int           tmcomp      P((register const struct tm * const atmp,
+                                    register const struct tm * const btmp));
+static time_t        time2       P((struct tm * const	tmp, void (* const
+                                   (funcp)(time_t, const long, struct tm *)),
+                                   const long offset, int * const okayp));
+static time_t        time1       P((struct tm * const	tmp, void (* const
+                                    funcp)(time_t, const long, struct tm *),
+                                    const long offset));
+time_t               mktime      P((struct tm * const tmp));
+time_t               timegm      P((struct tm *const tmp));
 
 #ifdef ALL_STATE
 static struct state *	lclptr;
@@ -219,9 +234,14 @@ int			daylight = 0;
 time_t			altzone = 0;
 #endif /* defined ALTZONE */
 
+#ifdef __STDC__
+static long
+detzcode(const char * const codep)
+#else
 static long
 detzcode(codep)
 const char * const	codep;
+#endif
 {
 	register long	result;
 	register int	i;
@@ -232,8 +252,13 @@ const char * const	codep;
 	return result;
 }
 
+#ifdef __STDC__
+static void
+settzname(void)
+#else
 static void
 settzname()
+#endif
 {
 	register const struct state * const	sp = lclptr;
 	register int				i;
@@ -301,7 +326,7 @@ openFile (const char *name, char *buf, int size)
   int len = strlen (name) + 2;
   int fd, l;
 
-  for (prev = TZDIR; cur = strchr (prev, ':'); prev = cur + 1)
+  for (prev = TZDIR; (cur = strchr (prev, ':')); prev = cur + 1)
   {
     l = cur - prev;
 
@@ -341,10 +366,15 @@ openFile (const char *name, char *buf, int size)
   return fd;
 }
 
+#ifdef __STDC__
+static int
+tzload(register const char *name, register struct state * const sp)
+#else
 static int
 tzload(name, sp)
 register const char *		name;
 register struct state * const	sp;
+#endif
 {
 	register const char *	p;
 	register int		i;
@@ -465,9 +495,14 @@ static const int	year_lengths[2] = {
 ** character.
 */
 
+#ifdef __STDC__
+static const char *
+getzname(register const char *strp) 
+#else
 static const char *
 getzname(strp)
 register const char *	strp;
+#endif
 {
 	register char	c;
 
@@ -484,12 +519,18 @@ register const char *	strp;
 ** Otherwise, return a pointer to the first character not part of the number.
 */
 
+#ifdef __STDC__
+static const char *
+getnum(register const char *strp, int * const nump,
+    const int min, const int max)
+#else
 static const char *
 getnum(strp, nump, min, max)
 register const char *	strp;
 int * const		nump;
 const int		min;
 const int		max;
+#endif
 {
 	register char	c;
 	register int	num;
@@ -517,10 +558,15 @@ const int		max;
 ** of seconds.
 */
 
+#ifdef __STDC__
+static const char *
+getsecs(register const char *strp, long * const secsp)
+#else
 static const char *
 getsecs(strp, secsp)
 register const char *	strp;
 long * const		secsp;
+#endif
 {
 	int	num;
 
@@ -552,10 +598,15 @@ long * const		secsp;
 ** Otherwise, return a pointer to the first character not part of the time.
 */
 
+#ifdef __STDC__
+static const char *
+getoffset(register const char *strp, long * const offsetp)
+#else
 static const char *
 getoffset(strp, offsetp)
 register const char *	strp;
 long * const		offsetp;
+#endif
 {
 	register int	neg;
 
@@ -580,10 +631,15 @@ long * const		offsetp;
 ** Otherwise, return a pointer to the first character not part of the rule.
 */
 
+#ifdef __STDC__
+static const char *
+getrule(const char *strp, register struct rule * const rulep)
+#else
 static const char *
 getrule(strp, rulep)
 const char *			strp;
 register struct rule * const	rulep;
+#endif
 {
 	if (*strp == 'J') {
 		/*
@@ -634,12 +690,18 @@ register struct rule * const	rulep;
 ** calculate the Epoch-relative time that rule takes effect.
 */
 
+#ifdef __STDC__
+static time_t
+transtime(const time_t janfirst, const int year,
+    register const struct rule *const rulep, const long offset)
+#else
 static time_t
 transtime(janfirst, year, rulep, offset)
 const time_t				janfirst;
 const int				year;
 register const struct rule * const	rulep;
 const long				offset;
+#endif
 {
 	register int	leapyear;
 	register time_t	value;
@@ -647,6 +709,7 @@ const long				offset;
 	int		d, m1, yy0, yy1, yy2, dow;
 
 	leapyear = isleap(year);
+	value = janfirst;
 	switch (rulep->r_type) {
 
 	case JULIAN_DAY:
@@ -728,11 +791,17 @@ const long				offset;
 ** appropriate.
 */
 
+#ifdef __STDC__
 static int
-tzparse(name, sp, lastditch)
-const char *			name;
-register struct state * const	sp;
+tzparse(const char *name, register struct state * const sp,
+    const int lastditch)
+#else
+static int
+tzparse( name, sp, lastditch
+const char *		name;
+register struct state * const sp;
 const int			lastditch;
+#endif
 {
 	const char *			stdname;
 	const char *			dstname;
@@ -745,6 +814,7 @@ const int			lastditch;
 	register char *			cp;
 	register int			load_result;
 
+	dstname = NULL;
 	stdname = name;
 	if (lastditch) {
 		stdlen = strlen(name);	/* length of standard zone name */
@@ -932,19 +1002,29 @@ const int			lastditch;
 	return 0;
 }
 
+#ifdef __STDC__
+static void
+gmtload(struct state * const sp)
+#else
 static void
 gmtload(sp)
 struct state * const	sp;
+#endif
 {
 	if (tzload(GMT, sp) != 0)
 		(void) tzparse(GMT, sp, TRUE);
 }
 
+#ifdef __STDC__
+void
+tzset(void)
+#else
 void
 tzset()
+#endif
 {
 	register const char *	name;
-	void tzsetwall();
+	void tzsetwall(void);
 
 	name = getenv("TZ");
 	if (name == NULL) {
@@ -976,8 +1056,13 @@ tzset()
 	settzname();
 }
 
+#ifdef __STDC__
 void
-tzsetwall()
+tzsetwall(void)
+#else
+void
+tzsetwall(void)
+#endif
 {
 	lcl_is_set = TRUE;
 #ifdef ALL_STATE
@@ -1004,11 +1089,17 @@ tzsetwall()
 */
 
 /*ARGSUSED*/
+#ifdef __STDC__
+static void
+localsub(const time_t * const	timep, const long offset,
+    struct tm * const tmp)
+#else
 static void
 localsub(timep, offset, tmp)
 const time_t * const	timep;
 const long		offset;
 struct tm * const	tmp;
+#endif
 {
 	register struct state *	sp;
 	register const struct ttinfo *	ttisp;
@@ -1055,9 +1146,14 @@ struct tm * const	tmp;
 	__libc_lock_unlock(__libc_localtime_lock);
 }
 
+#ifdef __STDC__
+struct tm *
+localtime(const time_t * const timep)
+#else
 struct tm *
 localtime(timep)
 const time_t * const	timep;
+#endif
 {
 	static struct tm	tm;
 
@@ -1065,10 +1161,15 @@ const time_t * const	timep;
 	return &tm;
 }
 
+#ifdef __STDC__
+struct tm *
+localtime_r (const time_t * const timep, struct tm *tm)
+#else
 struct tm *
 localtime_r (timep, tm)
 const time_t * const	timep;
 struct tm *tm;
+#endif
 {
 
 	localsub(timep, 0L, tm);
@@ -1079,11 +1180,16 @@ struct tm *tm;
 ** gmtsub is to gmtime as localsub is to localtime.
 */
 
+#ifdef __STDC__
+static void
+gmtsub(const time_t * const timep, const long offset, struct tm * const	tmp)
+#else
 static void
 gmtsub(timep, offset, tmp)
 const time_t * const	timep;
 const long		offset;
 struct tm * const	tmp;
+#endif
 {
 	__libc_lock_lock(__libc_gmtime_lock);
 	if (!gmt_is_set) {
@@ -1117,19 +1223,28 @@ struct tm * const	tmp;
 #endif /* HAVE_TM_ZONE */
 }
 
+#ifdef __STDC__
+struct tm *
+gmtime_r (const time_t * const timep, struct tm *tm)
+#else
 struct tm *
 gmtime_r (timep, tm)
 const time_t * const	timep;
 struct tm *tm;
-
+#endif
 {
 	gmtsub(timep, 0L, tm);
 	return tm;
 }
 
+#ifdef __STDC__
+struct tm *
+gmtime(const time_t * const timep)
+#else
 struct tm *
 gmtime(timep)
 const time_t * const	timep;
+#endif
 {
 	static struct tm	tm;
 
@@ -1137,12 +1252,18 @@ const time_t * const	timep;
 	return &tm;
 }
 
+#ifdef __STDC__
+static void
+timesub(const time_t * const timep, const long offset,
+   register const struct state * const	sp, register struct tm * const tmp)
+#else
 static void
 timesub(timep, offset, sp, tmp)
 const time_t * const			timep;
 const long				offset;
 register const struct state * const	sp;
 register struct tm * const		tmp;
+#endif
 {
 	register const struct lsinfo *	lp;
 	register long			days;
@@ -1235,10 +1356,15 @@ register struct tm * const		tmp;
 ** A la X3J11
 */
 
+#ifdef __STDC__
+char *
+asctime_r(register const struct tm *timeptr, char *result)
+#else
 char *
 asctime_r(timeptr, result)
 register const struct tm *	timeptr;
 char	*result;
+#endif
 {
 	static const char	wday_name[DAYSPERWEEK][3] = {
 		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
@@ -1257,25 +1383,40 @@ char	*result;
 	return result;
 }
 
+#ifdef __STDC__
+char *
+asctime(register const struct tm *timeptr)
+#else
 char *
 asctime(timeptr)
 register const struct tm *	timeptr;
+#endif
 {
 	static char	result[26];
 	return asctime_r (timeptr, result);
 }
 
+#ifdef __STDC__
+char *
+ctime(const time_t * const	timep)
+#else
 char *
 ctime(timep)
 const time_t * const	timep;
+#endif
 {
 	return asctime(localtime(timep));
 }
 
+#ifdef __STDC__
+char *
+ctime_r(const time_t * const timep, char *result)
+#else
 char *
 ctime_r(timep, result)
 const time_t * const	timep;
 char *result;
+#endif
 {
 	struct tm tm;
 	return asctime_r (localtime_r (timep, &tm), result);
@@ -1294,11 +1435,16 @@ char *result;
 #define WRONG	(-1)
 #endif /* !defined WRONG */
 
+#ifdef __STDC__
+static void
+normalize(int * const tensptr, int * const unitsptr, const int	base)
+#else
 static void
 normalize(tensptr, unitsptr, base)
 int * const	tensptr;
 int * const	unitsptr;
 const int	base;
+#endif
 {
 	if (*unitsptr >= base) {
 		*tensptr += *unitsptr / base;
@@ -1312,10 +1458,16 @@ const int	base;
 			/* assert (0 <= *unitsptr && *unitsptr < base); */
 }
 
+#ifdef __STDC__
+static int
+tmcomp(register const struct tm * const atmp,
+       register const struct tm * const btmp)
+#else
 static int
 tmcomp(atmp, btmp)
 register const struct tm * const atmp;
 register const struct tm * const btmp;
+#endif
 {
 	register int	result;
 
@@ -1328,12 +1480,19 @@ register const struct tm * const btmp;
 	return result;
 }
 
+#ifdef __STDC__
+static time_t
+time2(struct tm * const	tmp, 
+    void (* const  (funcp)(time_t, const long, struct tm *)),
+    const long offset, int * const okayp)
+#else
 static time_t
 time2(tmp, funcp, offset, okayp)
 struct tm * const	tmp;
-void (* const		funcp)();
+void (* const  (funcp)(time_t, const long, struct tm *));
 const long		offset;
 int * const		okayp;
+#endif
 {
 	register const struct state *	sp;
 	register int			dir;
@@ -1384,7 +1543,7 @@ int * const		okayp;
 	*/
 	t = (t < 0) ? 0 : ((time_t) 1 << bits);
 	for ( ; ; ) {
-		(*funcp)(&t, offset, &mytm);
+		(*funcp)((time_t)&t, (long)offset, &mytm);
 		dir = tmcomp(&mytm, &yourtm);
 		if (dir != 0) {
 			if (bits-- < 0)
@@ -1405,7 +1564,7 @@ int * const		okayp;
 		** gets checked.
 		*/
 		sp = (const struct state *)
-			((funcp == localsub) ? lclptr : gmtptr);
+			(((void *)funcp == (void *)localsub) ? lclptr : gmtptr);
 #ifdef ALL_STATE
 		if (sp == NULL)
 			return WRONG;
@@ -1418,7 +1577,7 @@ int * const		okayp;
 					continue;
 				newt = t + sp->ttis[j].tt_gmtoff -
 					sp->ttis[i].tt_gmtoff;
-				(*funcp)(&newt, offset, &mytm);
+				(*funcp)((time_t)&newt, (long)offset, &mytm);
 				if (tmcomp(&mytm, &yourtm) != 0)
 					continue;
 				if (mytm.tm_isdst != yourtm.tm_isdst)
@@ -1434,16 +1593,23 @@ int * const		okayp;
 	}
 label:
 	t += saved_seconds;
-	(*funcp)(&t, offset, tmp);
+	(*funcp)((time_t)&t, (long)offset, tmp);
 	*okayp = TRUE;
 	return t;
 }
 
+#ifdef __STDC__
+static time_t
+time1(struct tm * const	tmp,
+    void (* const funcp)(time_t, const long, struct tm *),
+    const long offset)
+#else
 static time_t
 time1(tmp, funcp, offset)
 struct tm * const	tmp;
-void (* const		funcp)();
+void (* const	funcp)(time_t, const long, struct tm *));
 const long		offset;
+#endif
 {
 	register time_t			t;
 	register const struct state *	sp;
@@ -1452,7 +1618,7 @@ const long		offset;
 
 	if (tmp->tm_isdst > 1)
 		tmp->tm_isdst = 1;
-	t = time2(tmp, funcp, offset, &okay);
+	t = time2(tmp, (void *)funcp, offset, &okay);
 	if (okay || tmp->tm_isdst < 0)
 		return t;
 	/*
@@ -1461,7 +1627,8 @@ const long		offset;
 	** We try to divine the type they started from and adjust to the
 	** type they need.
 	*/
-	sp = (const struct state *) ((funcp == localsub) ? lclptr : gmtptr);
+	sp = (const struct state *) (((void *)funcp
+                              == (void *)localsub) ? lclptr : gmtptr);
 #ifdef ALL_STATE
 	if (sp == NULL)
 		return WRONG;
@@ -1475,7 +1642,7 @@ const long		offset;
 			tmp->tm_sec += sp->ttis[otheri].tt_gmtoff -
 					sp->ttis[samei].tt_gmtoff;
 			tmp->tm_isdst = !tmp->tm_isdst;
-			t = time2(tmp, funcp, offset, &okay);
+			t = time2(tmp, (void *)funcp, offset, &okay);
 			if (okay)
 				return t;
 			tmp->tm_sec -= sp->ttis[otheri].tt_gmtoff -
@@ -1486,17 +1653,27 @@ const long		offset;
 	return WRONG;
 }
 
+#ifdef __STDC__
+time_t
+mktime(struct tm * const tmp)
+#else
 time_t
 mktime(tmp)
 struct tm * const	tmp;
+#endif
 {
-	return time1(tmp, localsub, 0L);
+	return time1(tmp, (void *)localsub, 0L);
 }
 
+#ifdef __STDC__
+time_t
+timegm (struct tm *const tmp)
+#else
 time_t
 timegm (tmp)
 struct tm *const tmp;
+#endif
 {
 	tmp->tm_isdst = 0;
-	return time1(tmp, gmtsub, 0L);
+	return time1(tmp, (void *)gmtsub, 0L);
 }
